@@ -9,6 +9,7 @@
 #include "3d_math.h"
 #include "adas.h"
 
+
 //------------------------------------------------------------------------------------------------
 // libraries
 //------------------------------------------------------------------------------------------------
@@ -96,6 +97,8 @@ char Last_Warning[256];         // text of last warning
 int bLast_Warning;              // last warning set
 char bDevice = 0;               // was device found?
 
+static char sound_dir[MAX_FILENAME];
+
 //------------------------------------------------------------------------------------------------
 // function declarations
 //------------------------------------------------------------------------------------------------
@@ -211,7 +214,7 @@ void adas_Init(ADAS_INIT_DATA * p_adas_data)
 
   memcpy(&ADAS_data, p_adas_data, sizeof(ADAS_INIT_DATA));
 
-  p_Device = alcOpenDevice((ALCubyte *) p_adas_data->Implementation);
+  p_Device = alcOpenDevice(p_adas_data->Implementation);
   if (p_Device) {
     bDevice = 1;
     p_Context = alcCreateContext(p_Device, NULL);
@@ -219,14 +222,14 @@ void adas_Init(ADAS_INIT_DATA * p_adas_data)
     if (p_Context) {
       //Set active context
       if (alcMakeContextCurrent(p_Context)) {
-        int bEAXExtPresent = alIsExtensionPresent((ALubyte *) "EAX2.0");
+        int bEAXExtPresent = alIsExtensionPresent("EAX2.0");
 
         if (!bEAXExtPresent) {
           adas_Set_Last_Warning("No EAX2.0 Extension found");
         }
         else {
-          p_EAXSet = (EAXSet) alGetProcAddress((ALubyte *) "EAXSet");
-          p_EAXGet = (EAXGet) alGetProcAddress((ALubyte *) "EAXGet");
+          p_EAXSet = (EAXSet) alGetProcAddress("EAXSet");
+          p_EAXGet = (EAXGet) alGetProcAddress("EAXGet");
 
           if (!p_EAXSet || !p_EAXGet) {
             adas_Set_Last_Error("Cannot find EAXSet and/or EAXGet function");
@@ -367,7 +370,12 @@ unsigned long adas_Load_First(char *p_Index_File, char *p_File_Name)
     return 0;
   }
 
-  fgets(data, 100, file);
+  if (fgets(data, 100, file) == NULL) {
+    adas_Set_Last_Error("Unable to read index file");
+    fclose(file);
+    file = NULL;
+    return 0;
+  }
   Size_of_Indexes = atoi(data);
   WaveFile = (ADAS_WAVEFILEDESC *) malloc(Size_of_Indexes * sizeof(ADAS_WAVEFILEDESC));
   if (!WaveFile) {
@@ -378,10 +386,24 @@ unsigned long adas_Load_First(char *p_Index_File, char *p_File_Name)
   }
 
   for (i = 0; i < Size_of_Indexes; i++) {
-    fgets(data, 100, file);
+    if (fgets(data, 100, file) == NULL) {
+      adas_Set_Last_Error("Unable to read index file");
+      fclose(file);
+      file = NULL;
+      free(WaveFile);
+      WaveFile = NULL;
+      return 0;
+    }
     newline_cut(data);
     strcpy(WaveFile[i].Name, data);
-    fgets(data, 100, file);
+    if (fgets(data, 100, file) == NULL) {
+      adas_Set_Last_Error("Unable to read index file");
+      fclose(file);
+      file = NULL;
+      free(WaveFile);
+      WaveFile = NULL;
+      return 0;
+    }
     WaveFile[i].Index = atoi(data);
   }
   fclose(file);
@@ -406,13 +428,13 @@ unsigned long adas_Load_First(char *p_Index_File, char *p_File_Name)
       return 0;
     }
 
-    alutLoadWAVFile(p_File_Name, &pSound->Format, &pSound->Data,
+    adasLoadWAVFile(p_File_Name, &pSound->Format, &pSound->Data,
                     &pSound->Size, &pSound->Frequece, &loop);
 
     alBufferData(pSound->Buffer, pSound->Format, pSound->Data, pSound->Size,
                  pSound->Frequece);
 
-    alutUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
+    adasUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
                   pSound->Frequece);
 
     pSound->Wave_Index = adas_Translate_Wave(p_File_Name);
@@ -455,7 +477,12 @@ unsigned long adas_Load_FirstMemory(char *p_Index_File, void *p_File,
   }
 
 
-  fgets(data, 100, file);
+  if (fgets(data, 100, file) == NULL) {
+    adas_Set_Last_Error("Unable to read index file");
+    fclose(file);
+    file = NULL;
+    return 0;
+  }
   Size_of_Indexes = atoi(data);
   WaveFile = (ADAS_WAVEFILEDESC *) malloc(Size_of_Indexes *
                                   sizeof(ADAS_WAVEFILEDESC));
@@ -468,10 +495,24 @@ unsigned long adas_Load_FirstMemory(char *p_Index_File, void *p_File,
   ZeroMemory(WaveFile, Size_of_Indexes*sizeof(ADAS_WAVEFILEDESC));
 
   for (i = 0; i < Size_of_Indexes; i++) {
-    fgets(data, 100, file);
+    if (fgets(data, 100, file) == NULL) {
+      adas_Set_Last_Error("Unable to read index file");
+      fclose(file);
+      file = NULL;
+      free(WaveFile);
+      WaveFile = NULL;
+      return 0;
+    }
     newline_cut(data);
     strcpy(WaveFile[i].Name, data);
-    fgets(data, 100, file);
+    if (fgets(data, 100, file) == NULL) {
+      adas_Set_Last_Error("Unable to read index file");
+      fclose(file);
+      file = NULL;
+      free(WaveFile);
+      WaveFile = NULL;
+      return 0;
+    }
     WaveFile[i].Index = atoi(data);
   }
   fclose(file);
@@ -495,7 +536,7 @@ unsigned long adas_Load_FirstMemory(char *p_Index_File, void *p_File,
   if (adasLoadWAVMemory(p_File, File_Size, &pSound->Format, &pSound->Data, &pSound->Size, &pSound->Frequece, &loop)) {
     alBufferData(pSound->Buffer, pSound->Format, pSound->Data, pSound->Size,
                  pSound->Frequece);
-    alutUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
+    adasUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
                   pSound->Frequece);
     pSound->Wave_Index = adas_Translate_Wave(p_File_Name);
     Size_of_Sound_Data++;
@@ -523,7 +564,8 @@ unsigned long adas_Load_Next(char *p_File_Name)
   if ((Size_of_Sound_Data > SIZEOFSOUNDDATA) || (!p_File_Name))
     return 0;
 
-  chdir(ADAS_data.Sound_Dir);
+  if (chdir(sound_dir))
+    return 0;
   file = fopen(p_File_Name, "r");
   if (file) {
     GetFileSize(file, &Return);
@@ -542,13 +584,13 @@ unsigned long adas_Load_Next(char *p_File_Name)
       return 0;
     }
 
-    alutLoadWAVFile(p_File_Name, &pSound->Format, &pSound->Data,
+    adasLoadWAVFile(p_File_Name, &pSound->Format, &pSound->Data,
                     &pSound->Size, &pSound->Frequece, &loop);
 
     alBufferData(pSound->Buffer, pSound->Format, pSound->Data, pSound->Size,
                  pSound->Frequece);
 
-    alutUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
+    adasUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
                   pSound->Frequece);
 
     pSound->Wave_Index = adas_Translate_Wave(p_File_Name);
@@ -590,7 +632,7 @@ unsigned long adas_Load_NextMemory(void *p_File, long File_Size,
   if (adasLoadWAVMemory(p_File, File_Size, &pSound->Format, &pSound->Data, &pSound->Size, &pSound->Frequece, &loop)) {
     alBufferData(pSound->Buffer, pSound->Format, pSound->Data, pSound->Size,
                  pSound->Frequece);
-    alutUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
+    adasUnloadWAV(pSound->Format, pSound->Data, pSound->Size,
                   pSound->Frequece);
     pSound->Wave_Index = adas_Translate_Wave(p_File_Name);
     Size_of_Sound_Data++;
@@ -911,11 +953,13 @@ int adas_Load_Wave(ADAS_SOUND_SOURCE * p_ss)
     return 0;
 
   strcpy(text, name);
-  chdir(ADAS_data.Sound_Dir);
-  alutLoadWAVFile(text, &Format, &Data, &Size, &Frequece, &loop);
-  alBufferData(p_ss->Buffer[p_ss->Buffer_Pointer], Format, Data, Size,
+  if (chdir(sound_dir))
+    return 0;
+  if (!adasLoadWAVFile(text, &Format, &Data, &Size, &Frequece, &loop))
+    return 0;
+  alBufferData(p_ss->Buffer[(int)p_ss->Buffer_Pointer], Format, Data, Size,
                Frequece);
-  alutUnloadWAV(Format, Data, Size, Frequece);
+  adasUnloadWAV(Format, Data, Size, Frequece);
   return 1;
 }
 
@@ -1585,11 +1629,11 @@ void * adas_ManagerProc(void *lpParameter)
 //------------------------------------------------------------------------------------------------
 void adas_Shot_Down_Manager(void)
 {
+  dword Exit_Code;
+
   if (!bDevice)
     return;
-// TODO
-#ifdef WIDOWS
-  unsigned long Exit_Code;
+
   Manager.Shot_down = 1;
   Sleep(500);
   GetExitCodeThread(Manager.Thread, &Exit_Code);
@@ -1604,7 +1648,6 @@ void adas_Shot_Down_Manager(void)
     Manager.Thread = 0;
     Manager.ThreadID = 0;
   }
-#endif
 }
 
 #ifdef WIDOWS
@@ -2565,6 +2608,23 @@ ALCdevice *adas_Get_Device(void)
   return p_Device;
 }
 
+void *
+adasLoadWAVFile(const char * filename, ALenum * format,
+		void **data, ALsizei * size, ALuint * frequency,
+		ALboolean * loop)
+{
+  ALfloat freq;
+  *data = alutLoadMemoryFromFile(filename, format, size, &freq);
+  if (!(*data)) {
+    fprintf(stderr, "ADAS: adasLoadWAVFile(): %s\n",
+	    alutGetErrorString(alutGetError()));
+    return (FALSE);
+  }
+  *frequency = (ALuint)freq;
+  *loop = AL_FALSE;
+  return (*data);
+}
+
 void *
 adasLoadWAVMemory(ALbyte * buffer, ALsizei buffer_length, ALenum * format,
                   void **data, ALsizei * size, ALuint * frequency,
@@ -2581,3 +2641,14 @@ adasLoadWAVMemory(ALbyte * buffer, ALsizei buffer_length, ALenum * format,
   *loop = AL_FALSE;
   return (*data);
 }
+
+void
+adasUnloadWAV(ALenum format, ALvoid * data, ALsizei size, ALsizei frequency)
+{
+  free(data);
+}
+
+void adas_set_sound_dir(char *p_dir)
+{
+  strcpy(sound_dir, p_dir);
+}
