@@ -8,9 +8,7 @@
 typedef struct
 {
   char no_sound;
-  char no_menu;
   char camera_type;
-  char log_file[256];
   char fullscreen;
   char ditering;  
   char text_detail;             //Texture-level - Uroven detailu textur (0 = max)
@@ -25,7 +23,6 @@ typedef struct
   int  key[POCETKLAVES];
   char debug_vypisy;
   char debug_light;
-  char pohled_berusky;
   char kvalita_castic;
   char kvalita_casticv;
   char kvalita_casticp;
@@ -38,15 +35,13 @@ typedef struct
   float text_ostrost;
   float p_kamera_alfa;
   float p_kamera_radius;
-  char init_visibility;
   char cursor[32];
-  char hires_textury;
-  char zrcado_aktivni;
+  char mirror_effects;
   char debug_kamery;
   char text_ans;  
   int  text_ans_stupen;
-  int  start_zpruhlednovani;
-  int  start_zvyraznovani;
+  int  bugs_highlight;
+  int  items_highlight;
   int  ovladani_rohy;
   int  ovladani_rohy_default;
   int  ovladani_rohy_smer;
@@ -59,4 +54,6 @@ typedef struct
 void Load_ini(void);
 void Save_ini(void);
 
+extern SETUP setup;
+
 #endif
