@@ -359,7 +359,7 @@ void anmend_Lift(size_ptr param, size_ptr param2, size_ptr p_param)
   s = ap_Play_Sound(0,0,0, pos, p_itm->p_Object->Specific[0].Index, NULL, &ad);
 
   if (s >= 0) {
-		for(i=0;i<(abs(param)-1);i++)
+		for(i=0;i<(abs((int)param)-1);i++)
 			adas_Queue_Sound(s, p_itm->p_Object->Specific[1].Index);
 
 		adas_Queue_Sound(s, p_itm->p_Object->Specific[2].Index);
@@ -371,7 +371,7 @@ void anmend_Lift(size_ptr param, size_ptr param2, size_ptr p_param)
     if (s < 0)
       return;
 
-    p_Level->LiftParticles[s].dwStop = abs(param) * 250;
+    p_Level->LiftParticles[s].dwStop = abs((int)param) * 250;
   }
 }
 
@@ -584,10 +584,6 @@ void anmend_Steps(size_ptr param, size_ptr param2, size_ptr p_param)
     bOnWater = 1;
   }
 
-/*   if(p_set_anim->last && Level.bPosouvatKameru && 
-	  !(p_Level->bB1ExtraCameraRot && p_Level->bPohled_Berusky))
-		rani_privaz_kameru(p_set_anim->animation[p_set_anim->last-1].p_run);*/
-
   kam_pol_get((BOD *) pos, &fr, &fi, &vzdal);
 
   if (p_Level->bB1ExtraCameraRot && p_Level->bPohled_Berusky) {
