@@ -5,7 +5,6 @@
 #include "Berusky3d_kofola_interface.h"
 
 #include "controls.h"
-#include "Apak.h"
 #include "2D_graphic.h"
 #include "font.h"
 #include "menu_def.h"
@@ -14,7 +13,7 @@
 #define HDC2DD -1
 
 extern B2_FONT b2_2d_font;
-extern APAK_HANDLE *pControlsArchive;
+extern char pControlsDir[MAX_FILENAME];
 extern HDC BackDC;
 static char bBlockList = 0;
 static char bExclusive = 0;
@@ -34,49 +33,47 @@ void co_Del_Combo_List(COMBO_CONTROL * p_co, int hdc, int xcor, int ycor);
 
 int co_Load_Graphic(int combo_var)
 {
-  pControlsArchive->pActualNode = pControlsArchive->pRootNode->pNextNode;
-
   if (!combo_var) {
-    hdcCO.hdcCombo = ddxLoadBitmap("combo.bmp", pControlsArchive);
-    hdcCO.hdcComboEnd = ddxLoadBitmap("combo_end.bmp", pControlsArchive);
-    hdcCO.hdcComboMid = ddxLoadBitmap("combo_mid.bmp", pControlsArchive);
+    hdcCO.hdcCombo = ddxLoadBitmap("combo.png", pControlsDir);
+    hdcCO.hdcComboEnd = ddxLoadBitmap("combo_end.png", pControlsDir);
+    hdcCO.hdcComboMid = ddxLoadBitmap("combo_mid.png", pControlsDir);
   }
   else {
-    hdcCO.hdcCombo = ddxLoadBitmap("combo_l.bmp", pControlsArchive);
-    hdcCO.hdcComboEnd = ddxLoadBitmap("combo_end_l.bmp", pControlsArchive);
-    hdcCO.hdcComboMid = ddxLoadBitmap("combo_mid_l.bmp", pControlsArchive);
+    hdcCO.hdcCombo = ddxLoadBitmap("combo_l.png", pControlsDir);
+    hdcCO.hdcComboEnd = ddxLoadBitmap("combo_end_l.png", pControlsDir);
+    hdcCO.hdcComboMid = ddxLoadBitmap("combo_mid_l.png", pControlsDir);
   }
 
-  hdcCO.hdcComboUp = ddxLoadBitmap("combo_up.bmp", pControlsArchive);
-  hdcCO.hdcComboDown = ddxLoadBitmap("combo_down.bmp", pControlsArchive);
-  hdcCO.hdcComboDrop = ddxLoadBitmap("combo_drop.bmp", pControlsArchive);
-  hdcCO.hdcComboMover = ddxLoadBitmap("combo_mover.bmp", pControlsArchive);
-  hdcCO.hdcComboMoverb = ddxLoadBitmap("combo_moverb.bmp", pControlsArchive);
+  hdcCO.hdcComboUp = ddxLoadBitmap("combo_up.png", pControlsDir);
+  hdcCO.hdcComboDown = ddxLoadBitmap("combo_down.png", pControlsDir);
+  hdcCO.hdcComboDrop = ddxLoadBitmap("combo_drop.png", pControlsDir);
+  hdcCO.hdcComboMover = ddxLoadBitmap("combo_mover.png", pControlsDir);
+  hdcCO.hdcComboMoverb = ddxLoadBitmap("combo_moverb.png", pControlsDir);
 
   if (!combo_var)
-    hdcED.hdcEdit = ddxLoadBitmap("edit.bmp", pControlsArchive);
+    hdcED.hdcEdit = ddxLoadBitmap("edit.png", pControlsDir);
   else
-    hdcED.hdcEdit = ddxLoadBitmap("edit_l.bmp", pControlsArchive);
+    hdcED.hdcEdit = ddxLoadBitmap("edit_l.png", pControlsDir);
 
-  hdcLI.hdcListDown = ddxLoadBitmap("list_down.bmp", pControlsArchive);
-  hdcLI.hdcListMover = ddxLoadBitmap("list_mover.bmp", pControlsArchive);
-  hdcLI.hdcListUp = ddxLoadBitmap("list_up.bmp", pControlsArchive);
-  hdcLI.hdcListTop = ddxLoadBitmap("list_top.bmp", pControlsArchive);
-  hdcLI.hdcListBottom = ddxLoadBitmap("list_bottom.bmp", pControlsArchive);
-  hdcLI.hdcListLeft = ddxLoadBitmap("list_left.bmp", pControlsArchive);
-  hdcLI.hdcListRight = ddxLoadBitmap("list_right.bmp", pControlsArchive);
-  hdcLI.hdcListTL = ddxLoadBitmap("list_tl.bmp", pControlsArchive);
-  hdcLI.hdcListTR = ddxLoadBitmap("list_tr.bmp", pControlsArchive);
-  hdcLI.hdcListBL = ddxLoadBitmap("list_bl.bmp", pControlsArchive);
-  hdcLI.hdcListBR = ddxLoadBitmap("list_br.bmp", pControlsArchive);
+  hdcLI.hdcListDown = ddxLoadBitmap("list_down.png", pControlsDir);
+  hdcLI.hdcListMover = ddxLoadBitmap("list_mover.png", pControlsDir);
+  hdcLI.hdcListUp = ddxLoadBitmap("list_up.png", pControlsDir);
+  hdcLI.hdcListTop = ddxLoadBitmap("list_top.png", pControlsDir);
+  hdcLI.hdcListBottom = ddxLoadBitmap("list_bottom.png", pControlsDir);
+  hdcLI.hdcListLeft = ddxLoadBitmap("list_left.png", pControlsDir);
+  hdcLI.hdcListRight = ddxLoadBitmap("list_right.png", pControlsDir);
+  hdcLI.hdcListTL = ddxLoadBitmap("list_tl.png", pControlsDir);
+  hdcLI.hdcListTR = ddxLoadBitmap("list_tr.png", pControlsDir);
+  hdcLI.hdcListBL = ddxLoadBitmap("list_bl.png", pControlsDir);
+  hdcLI.hdcListBR = ddxLoadBitmap("list_br.png", pControlsDir);
 
-  hdcBU.hdcButtonL = ddxLoadBitmap("button_l.bmp", pControlsArchive);
-  hdcBU.hdcButtonS = ddxLoadBitmap("button_s.bmp", pControlsArchive);
-  hdcCH.hdcCheck = ddxLoadBitmap("checkbox.bmp", pControlsArchive);
-  hdcCH.hdcGray = ddxLoadBitmap("gray.bmp", pControlsArchive);
-  hdcPR.hdcLine = ddxLoadBitmap("progres_line.bmp", pControlsArchive);
-  hdcPR.hdcMover = ddxLoadBitmap("progres_mover.bmp", pControlsArchive);
-  hdcFR.hdcFrame = ddxLoadBitmap("frame.bmp", pControlsArchive);
+  hdcBU.hdcButtonL = ddxLoadBitmap("button_l.png", pControlsDir);
+  hdcBU.hdcButtonS = ddxLoadBitmap("button_s.png", pControlsDir);
+  hdcCH.hdcCheck = ddxLoadBitmap("checkbox.png", pControlsDir);
+  hdcCH.hdcGray = ddxLoadBitmap("gray.png", pControlsDir);
+  hdcPR.hdcLine = ddxLoadBitmap("progres_line.png", pControlsDir);
+  hdcPR.hdcMover = ddxLoadBitmap("progres_mover.png", pControlsDir);
+  hdcFR.hdcFrame = ddxLoadBitmap("frame.png", pControlsDir);
 
   bBlockList = 0;
   bExclusive = 0;
@@ -2622,7 +2619,7 @@ void co_delete(char *cfile)
 
 	_getcwd(olddir, MAX_FILENAME);
 
-	GetPrivateProfileString("game","save_dir","c:\\",dir,MAX_FILENAME,ini_file);
+	GetPrivateProfileString("install","save_dir","/",dir,MAX_FILENAME,ini_file);
   working_file_translate(dir,256);
 
 	chdir((dir));
@@ -3065,6 +3062,9 @@ int co_Handle_Edit_Key_Filter(void)
 
   //GetKeyboardState(k);
 
+  if (isprint(key_pressed))
+    return 0;
+
   if (key[K_ESC])
     c++;
 
@@ -3218,7 +3218,7 @@ int co_Handle_Edit(CONTROL_EDIT * p_ed, int x, int y, int hdc, int xcor, int yco
       int   size;
 
       if (co_Handle_Edit_Key_Filter() || (co_Handle_wsclen(p_ed) && !key[K_BKSP])) {
-        memset(key, 0, POCET_KLAVES * sizeof(char));
+        memset(key, 0, POCET_KLAVES * sizeof(int));
         return 1;
       }
 
@@ -3226,18 +3226,18 @@ int co_Handle_Edit(CONTROL_EDIT * p_ed, int x, int y, int hdc, int xcor, int yco
       size = MultiByteToWideChar(CP_ACP, 0, (char *) &key_pressed, 1, wt, sizeof(wt)/sizeof(wt[0]));
       if(size > 1) {
         // we have got some unsupported character - ignore it
-        memset(key, 0, POCET_KLAVES * sizeof(char));
+        memset(key, 0, POCET_KLAVES * sizeof(int));
         return 1;
       }
       wt[0] = co_ToUnicode(key_pressed);
 
       if (!wt[0]) {
-        memset(key, 0, POCET_KLAVES * sizeof(char));
+        memset(key, 0, POCET_KLAVES * sizeof(int));
         return 1;
       }
 
       if (wt[0] == '_') {
-        memset(key, 0, POCET_KLAVES * sizeof(char));
+        memset(key, 0, POCET_KLAVES * sizeof(int));
         return 1;
       }
 
@@ -3280,7 +3280,7 @@ int co_Handle_Edit(CONTROL_EDIT * p_ed, int x, int y, int hdc, int xcor, int yco
 
       p_ed->tx = draw_edit(p_ed, x, y, hdc, xcor, ycor);
 
-      memset(key, 0, POCET_KLAVES * sizeof(char));
+      memset(key, 0, POCET_KLAVES * sizeof(int));
     }
   }
 
