@@ -1,5 +1,5 @@
 #include <stdio.h>
-#include "Apak.h"
+#include <errno.h>
 #include "font3d.h"
 //#include "2d_graphic.h"
 //#include "2ddx.h"
@@ -166,7 +166,7 @@ int fn2_Set_Font_Bmps(GAME_TRIGER * gt, TRIGER_STRUCTURE * ts)
                   Value]), text, 256, NULL, NULL);
 
             b2_3d_font.iBitmap[gt->command[i].Parametr[0].Value] =
-              ddx2LoadBitmap(text, b2_3d_font.pArchive);
+              ddx2LoadBitmap(text, b2_3d_font.dir);
           }
         }
         break;
@@ -183,7 +183,7 @@ int fn2_Set_Font_Bmps(GAME_TRIGER * gt, TRIGER_STRUCTURE * ts)
                   Value]), text, 256, NULL, NULL);
 
 
-            txt_nahraj_texturu_z_func(b2_3d_font.pArchive, text,
+            txt_nahraj_texturu_z_func(b2_3d_font.dir, text,
               &b2_3d_font.tex[gt->command[i].Parametr[0].Value], 1, 0,
               &b2_3d_font.konf[gt->command[i].Parametr[0].Value], bmp_nahraj);
           }
@@ -371,7 +371,7 @@ void fn2_Draw_MessageRECT(int iSurface, int iXpos, int iYpos,
 
       *iYmax = y;
 
-      if (!end_char(wtext[i - 1])) {
+      if (i && !end_char(wtext[i - 1])) {
         tWord[iWordl].xEnd = x - b2_3d_font.iXPlus;
         tWord[iWordl].wLen = &wtext[i] - tWord[iWordl].wcWord;
         iWordl++;
@@ -627,73 +627,25 @@ void fn2_Gen_Texture(byte ** lpTexture, int iXSize, int iYSize, int iXpos,
   }
 }
 
-int fn2_Open_Archive(char *cFile, APAK_HANDLE ** pAHandle, char *cAppName,
-  char *cKeyName)
-{
-  int e;
-  char text[MAX_FILENAME];
-
-  GetPrivateProfileString(cAppName, cKeyName, "c:\\", text, MAX_FILENAME, ini_file);
-  working_file_translate(text, MAX_FILENAME);
-  chdir((text));
-
-  (*pAHandle) = apakopen(cFile, text, &e);
-
-  if (!(*pAHandle)) {
-    kprintf(1, "Unable to open archive %s", cFile);
-
-    switch (e) {
-      case APAK_FILE_NOT_FOUND:
-        kprintf(1, "Reason: File not found");
-        break;
-      case APAK_UNABLE_TO_READ:
-        kprintf(1, "Reason: Unable to read from file");
-        break;
-      case APAK_VERSION_MISMATCH:
-        kprintf(1, "Reason: Version mismatch");
-        break;
-      case APAK_OUT_OF_MEMORY:
-        kprintf(1, "Reason: Out of memory");
-        break;
-    }
-
-    assert(0);
-    abort();
-  }
-
-  achdir((*pAHandle), text);
-
-  kprintf(1, "APAK: %s", cFile);
-  kprintf(1, "Velikost AFAT: %.1fKB",
-    (*pAHandle)->FileHeader.apuISizeofFAT / 1000.0f);
-  kprintf(1, "Velikost Archivu: %.1fMB",
-    (*pAHandle)->FileHeader.apuLSizeofPAK / 1000000.0f);
-  kprintf(1, "Souboru: %d", (*pAHandle)->FileHeader.apuICountofFiles);
-  kprintf(1, "Adresaru: %d", (*pAHandle)->FileHeader.apuICountofDirectiories);
-  kprintf(1, "Uzlu: %d", (*pAHandle)->FileHeader.apuICountofNodes);
-
-  return 1;
-}
-
 int fn2_Load_Grammar(char *pFile, GRAMMAR * pGr)
 {
   FILE *file;
-  char text[256];
+  char text[MAX_FILENAME];
 
-  file = aopen(b2_3d_font.pArchive, pFile, "rb");
+  construct_path(text, MAX_FILENAME, 2, b2_3d_font.dir, pFile);
+  file = fopen(text, "rb");
 
   if (!file)
     return 0;
 
   pGr->LastMask = 0;
 
-  while (!aeof(file)) {
-    agets(text, 256, file);
+  while (fgets(text, 256, file)) {
     gr_Add_Mask(text, pGr);
     strcpy(text, "");
   }
 
-  aclose(file);
+  fclose(file);
   return 1;
 }
 
@@ -701,60 +653,72 @@ char fn2_Load_Triger(char *pFile, GAME_TRIGER * pTriger, GRAMMAR * pGr,
   TRIGER_STRUCTURE * pTStruct)
 {
   FILE *file;
-  WCHAR wtext[128];
-  word wotext[128];
+  char text[64];
+  WCHAR wtext[sizeof(text)*4];
+  char filename[MAX_FILENAME];
 
 
   pTriger->lastcommand = 0;
 
-  file = aopen(b2_3d_font.pArchive, pFile, "rb");
-
-  aunicode(file);
+  construct_path(filename, MAX_FILENAME, 2, b2_3d_font.dir, pFile);
+  file = fopen(filename, "rb");
 
   if (!file)
     return 0;
 
-  aseek(file, 2, SEEK_SET);
-
-  while (!aeof(file)) {
-    memset(wtext, 0, 128 * sizeof(WCHAR));
-    if(agets((char *) wotext, 256, file)) {
-      wchar_windows_to_linux(wotext, 128, wtext);
+  while (!feof(file)) {
+    memset(wtext, 0, sizeof(wtext));
+    if (fgets(text, sizeof(text), file)) {
+      if (MultiByteToWideChar(CP_ACP, 0,
+                              text, sizeof(text)/sizeof(*text),
+                              wtext, sizeof(wtext)/sizeof(*wtext)) < 0) {
+        fclose(file);
+        return 0;
+      }
       trig_Parse_LineU(wtext, &pTriger->command[pTriger->lastcommand], pTriger,
                        pGr, pTStruct);
     }
   }
 
-  aclose(file);
+  fclose(file);
   return 1;
 }
 
-int fn2_Set_Font(char *cPAK)
+int fn2_Set_Font(char *cDir)
 {
   int i;
+  char filename[MAX_FILENAME];
+  long size;
+  int iTSize;
+  char *buffer;
 
   memset(&b2_3d_font, 0, sizeof(B2_FONT));
 
-  if (!fn2_Open_Archive(cPAK, &b2_3d_font.pArchive, "game", "bitmap_dir"))
+  construct_path(b2_3d_font.dir, MAX_FILENAME, 2, p_ber->dir.bitmap_dir, cDir);
+
+  construct_path(filename, MAX_FILENAME, 2, b2_3d_font.dir, "texts.txt");
+  b2_3d_font.file = fopen(filename, "rb");
+
+  if (!b2_3d_font.file)
     return 0;
 
-  b2_3d_font.file = aopen(b2_3d_font.pArchive, "texts.txt", "rb");
-
-  if (!b2_3d_font.file) {
-    apakclose(&b2_3d_font.pArchive);
+  if (fseek(b2_3d_font.file, 0, SEEK_END) < 0 ||
+      (size = ftell(b2_3d_font.file)) < 0 ||
+      fseek(b2_3d_font.file, 0, SEEK_SET) < 0)
     return 0;
-  }
-  else {
-    apuInt iTSize;
 
-    agetbuffer(b2_3d_font.file, (char **) &b2_3d_font.pTBuffer, &iTSize);
-    b2_3d_font.pTBuffer = wchar_windows_to_linux((word *) b2_3d_font.pTBuffer, iTSize);
-
-    if (!b2_3d_font.pTBuffer) {
-      apakclose(&b2_3d_font.pArchive);
-      return 0;
-    }
-  }
+  buffer = (char *) mmalloc(sizeof(*buffer) * (size + 1));
+  if (fread(buffer, sizeof(*buffer),
+            size, b2_3d_font.file) != (size_t) size)
+    return 0;
+  buffer[size] = 0;
+  iTSize = MultiByteToWideChar(CP_ACP, 0, buffer, size, NULL, 0);
+  b2_3d_font.pTBuffer =
+    (WCHAR *) mmalloc(sizeof(*b2_3d_font.pTBuffer) * (iTSize + 1));
+  size = MultiByteToWideChar(CP_ACP, 0, buffer, size,
+                             b2_3d_font.pTBuffer, iTSize + 1);
+  assert(size == iTSize);
+  free(buffer);
 
   if (!fn2_Load_Grammar("font_grammar.txt", &b2_3d_font.gr))
     return 0;
@@ -780,7 +744,7 @@ int fn2_Set_Font(char *cPAK)
 
 int fn2_Load_Bitmaps(void)
 {
-  if (!b2_3d_font.pArchive)
+  if (b2_3d_font.dir[0] == '\0')
     return 0;
 
   kom_set_default_text_config(0, 0, 1, 0, 0, 1);
@@ -794,8 +758,8 @@ void fn2_Release_Font(void)
 {
   int i;
 
-  aclose(b2_3d_font.file);
-  apakclose(&b2_3d_font.pArchive);
+  fclose(b2_3d_font.file);
+  free(b2_3d_font.pTBuffer);
 
 //      if(_2dd.bitmap)
   for (i = 0; i < FONT_MAX_BMP; i++)
@@ -1154,7 +1118,11 @@ void fn2_Convert_Rect(char *cFile, int xmax, int ymax)
         {
           if (gt->command[i].LastParam > 5 &&
             gt->command[i].Parametr[0].Type == 3) {
-            fgets(textt, 256, fi);
+            if (fgets(textt, 256, fi) == NULL) {
+	      fclose(fi);
+	      fclose(f);
+	      return;
+	    }
             strncpy(t, textt, 10);
             t[10] = '\0';
             strcpy(text, "");
