@@ -598,11 +598,13 @@ void lsi_Load_Level_Script(LEVELINFO * p_Level, char *cFile)
   GAME_TRIGER gt;
   GRAMMAR gr;
 
-  chdir(DATA_DIR);
+  if (chdir(DATA_DIR))
+    return;
 
   gr_Load_Grammar("lsc_grammar.txt", &gr);
 
-  chdir(GAME_DATA_DIR);
+  if (chdir(GAME_DATA_DIR))
+    return;
 
   ts.LastStr = 0;
   ts.sizeofT = 0;
@@ -796,41 +798,52 @@ void lsi_Load_Level_Script(LEVELINFO * p_Level, char *cFile)
 }
 
 int lsi_Create_Level_Raw(char *p_Level_Name, BUNKA_LEVELU_DISK ** b_l_d,
-  int *size)
+                         int *size)
 {
   LEVEL_HEADER l_h;
   char text[MAX_FILENAME + 1];
+  char filename[MAX_FILENAME];
   FILE *file;
 
   kprintf(1, "Tvorba raw struktury pro komata...");
-  getcwd(text, MAX_FILENAME);
+  if (getcwd(text, MAX_FILENAME) == NULL) {
+    kprintf(1, "Cannot get current directory");
+    return 0;
+  }
   kprintf(1, "_getcwd = %s", text);
 
-  file = fopen(p_Level_Name, "rb");
+  strcpy(filename, p_Level_Name);
+  file = fopen(filename, "rb");
   if (!file) {
-    chdir((text));
-    sprintf(text, "Unable to find level %s", p_Level_Name);
-    kprintf(1, text);
-    return 0;
+    kprintf(1, "Unable to load level '%s', guessing level name...", filename);
+    
+    strcat(filename, KONCOVKA_LEVELU);
+  
+    file = fopen(filename, "rb");
+    if (!file) {
+      kprintf(1, "Unable to load level '%s'.", filename);
+      return 0;
+    }
+    kprintf(1, "Going with '%s' level name", filename);
   }
 
-  fread(&l_h, sizeof(LEVEL_HEADER), 1, file);
-
+  if (fread(&l_h, sizeof(LEVEL_HEADER), 1, file) != 1) {
+    fclose(file);
+    kprintf(1, "Unable to load level '%s'.", filename);
+    return 0;
+  }
   (*size) = l_h.x * l_h.y * l_h.z * 2;
-
-  (*b_l_d) =
-    (BUNKA_LEVELU_DISK *) malloc((*size) * sizeof(BUNKA_LEVELU_DISK));
-
-  if (!(*b_l_d)) {
-    kprintf(1, "Nepodarilo se naalokovat pamet na raw strukturu pro komata");
-    chdir((text));
+  (*b_l_d) = (BUNKA_LEVELU_DISK *) mmalloc((*size) * sizeof(BUNKA_LEVELU_DISK));
+  if (fread((*b_l_d), sizeof(BUNKA_LEVELU_DISK), (*size), file) != (size_t) (*size)) {
+    free(*b_l_d);
+    fclose(file);
+    kprintf(1, "Unable to load level '%s'.", filename);
     return 0;
   }
 
-  fread((*b_l_d), sizeof(BUNKA_LEVELU_DISK), (*size), file);
-
   fclose(file);
-  chdir((text));
+  if (chdir(text))
+    return 0;
   return 1;
 }
 
@@ -840,6 +853,7 @@ int lsi_Create_Level_Raw(char *p_Level_Name, BUNKA_LEVELU_DISK ** b_l_d,
 int lsi_Load_Level(char *p_Level_Name, LEVELINFO * p_Level)
 {
   char text[256];
+  char filename[MAX_FILENAME];
   FILE *file;
   int Vazba1, Vazba2, Rotation;
   int i, Guid, errors = 0, error;
@@ -864,8 +878,7 @@ int lsi_Load_Level(char *p_Level_Name, LEVELINFO * p_Level)
     p_Level->BeetleAnim[i].Flag = -1;
     p_Level->BeatleSmoke[i].iItem = -1;
     p_Level->BeatleSmoke[i].iStepc = 0;
-    ZeroMemory(&p_Level->BeatleSmoke[i].sStopy,
-      sizeof(SYSTEMKOUROVYCHCASTIC));
+    ZeroMemory(&p_Level->BeatleSmoke[i].sStopy, sizeof(SYSTEMKOUROVYCHCASTIC));
     p_Level->BeatleSmoke[i].sStopy.System = (size_ptr)NULL;
     p_Level->BeetleAnim[i].iAnimCount = 0;
     p_Level->BeetleAnim[i].iEffect = 0;
@@ -873,27 +886,42 @@ int lsi_Load_Level(char *p_Level_Name, LEVELINFO * p_Level)
     p_Level->BeetleAnim[i].dwTime = timeGetTime();
   }
 
-  getcwd(text, 255);
+  if (getcwd(text, 255) == NULL) {
+    kprintf(1, "Unable to get current directory");
+    return -1;
+  }
 
   kprintf(1, "Loading level from %s ...", text);
   
-  kvalita_castic = GetPrivateProfileInt("hra", "kvalita_castic", 0, ini_file);
+  kvalita_castic = GetPrivateProfileInt("game", "kvalita_castic", 0, ini_file);
   kvalita_castic++;
 
-  kvalita_casticv = GetPrivateProfileInt("hra", "kvalita_casticv", 0, ini_file);
+  kvalita_casticv = GetPrivateProfileInt("game", "kvalita_casticv", 0, ini_file);
   kvalita_casticv++;
 
   p_Level->KvalitaCastic = kvalita_castic;
   p_Level->KvalitaCasticV = kvalita_casticv;
 
-  file = fopen(p_Level_Name, "rb");
+  strcpy(filename, p_Level_Name);
+  file = fopen(filename, "rb");
   if (!file) {
-    sprintf(text, "Unable to find level %s", p_Level_Name);    
-    kprintf(1, text);
-    return -1;
+    kprintf(1, "Unable to load level '%s', guessing level name...", filename);
+  
+    strcat(filename, KONCOVKA_LEVELU);
+  
+    file = fopen(filename, "rb");
+    if (!file) {
+      kprintf(1, "Unable to load level '%s'.", filename);
+      return(-1);
+    }
+    kprintf(1, "Going with '%s' level name", filename);
   }
 
-  fread(&l_h, sizeof(LEVEL_HEADER), 1, file);
+  if (fread(&l_h, sizeof(LEVEL_HEADER), 1, file) != 1) {
+    fclose(file);
+    kprintf(1, "Unable to load level '%s'.", filename);
+    return 0;
+  }
 
   p_Level->Size[0] = l_h.x;
   p_Level->Size[1] = l_h.z;
@@ -1279,7 +1307,7 @@ void lsi_Release_Level(LEVELINFO * p_Level)
     p_Level->Rain.bWaterCircles = 0;
   }
 
-  memset(&p_Level->Rain, 0, sizeof(p_Level->Rain));
+  memset((void *) &p_Level->Rain, 0, sizeof(p_Level->Rain));
 
   if (p_Level->StreetRain.bWaterCircles) {
     kprintf(1, "p_Level->StreetRain.pWSystem");
@@ -1293,7 +1321,7 @@ void lsi_Release_Level(LEVELINFO * p_Level)
     p_Level->StreetRain.bWaterCircles = 0;
   }
 
-  memset(&p_Level->StreetRain, 0, sizeof(p_Level->StreetRain));
+  memset((void *) &p_Level->StreetRain, 0, sizeof(p_Level->StreetRain));
 
   if (p_Level->Snow.bSnow) {
     kprintf(1, "p_Level->Snow.pSystem");
@@ -1306,7 +1334,7 @@ void lsi_Release_Level(LEVELINFO * p_Level)
     p_Level->Snow.bSnow = 0;
   }
 
-  memset(&p_Level->Snow, 0, sizeof(p_Level->Snow));
+  memset((void *) &p_Level->Snow, 0, sizeof(p_Level->Snow));
 
   if (p_Level->Mist.bMist) {
     kprintf(1, "edl_svetlo_zrus ,p_Level->Mist.hSvetlo = %d",
@@ -1325,6 +1353,9 @@ void lsi_Get_Dir_Name(char *cText, char *cLevel)
     strncpy(cText, cLevel, length);
     cText[length] = 0;
   }
+  else {
+    strcpy(cText, cLevel);
+  }
 }
 
 void lsi_copy_save(char *cMask, LEVELINFO * p_Level)
@@ -1333,28 +1364,39 @@ void lsi_copy_save(char *cMask, LEVELINFO * p_Level)
   char cLevelMask[MAX_FILENAME];
   char csrc[MAX_FILENAME + 1];
   char cout[MAX_FILENAME + 1];
+  int ret;
 
   strcpy(dir, GAME_LEVEL_DIR);
   lsi_Get_Dir_Name(cLevelMask, p_Level->cLoadedFrom);
 
   sprintf(cout, "%s%s", cMask, ".b2l");
-  sprintf(csrc, "%s%c%s%c%s%s", dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".b2l");
+  ret = snprintf(csrc, sizeof(csrc), "%s%c%s%c%s%s",
+		 dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".b2l");
+  assert(ret < (int) sizeof(csrc));
   lsi_file_copy(csrc, cout);
 
   sprintf(cout, "%s%s", cMask, ".b2t");
-  sprintf(csrc, "%s%c%s%c%s%s", dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".b2t");
+  ret = snprintf(csrc, sizeof(csrc), "%s%c%s%c%s%s",
+		dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".b2t");
+  assert(ret < (int) sizeof(csrc));
   lsi_file_copy(csrc, cout);
 
   sprintf(cout, "%s%s", cMask, ".bpr");
-  sprintf(csrc, "%s%c%s%c%s%s", dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".bpr");
+  ret = snprintf(csrc, sizeof(csrc), "%s%c%s%c%s%s",
+		 dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".bpr");
+  assert(ret < (int) sizeof(csrc));
   lsi_file_copy(csrc, cout);
 
   sprintf(cout, "%s%s", cMask, ".lv6");
-  sprintf(csrc, "%s%c%s%c%s%s", dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".lv6");
+  ret = snprintf(csrc, sizeof(csrc), "%s%c%s%c%s%s",
+		 dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".lv6");
+  assert(ret < (int) sizeof(csrc));
   lsi_file_copy(csrc, cout);
 
   sprintf(cout, "%s%s", cMask, ".ply");
-  sprintf(csrc, "%s%c%s%c%s%s", dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".ply");
+  ret = snprintf(csrc, sizeof(csrc), "%s%c%s%c%s%s",
+		 dir, DIR_SLASH, cLevelMask, DIR_SLASH, cLevelMask, ".ply");
+  assert(ret < (int) sizeof(csrc));
   lsi_file_copy(csrc, cout);
 }
 
@@ -1364,11 +1406,12 @@ int lsi_Get_Save_Info(char *p_Level_Name, int *pActLevel, int *pActScene)
   char text[256];
   PLAYER_PROFILE pPlayer;
   LEVEL_HEADER l_h;
-  WCHAR wTmp[512];
+  WCHAR wTmp[32];
+  char cTmp[256+1];
   int ver;
 
-  chdir(SAVE_DIR);
-  chdir(p_Level_Name);
+  if (chdir(SAVE_DIR) || chdir(p_Level_Name))
+    return 0;
 
   sprintf(text, "%s.lvc", p_Level_Name);
 
@@ -1377,11 +1420,14 @@ int lsi_Get_Save_Info(char *p_Level_Name, int *pActLevel, int *pActScene)
   if (!file)
     return 0;
 
-  fread(&pPlayer, sizeof(PLAYER_PROFILE), 1, file);
-  fread(wTmp, 32 * sizeof(WCHAR), 1, file);
-  fread(&ver, sizeof(int), 1, file);
-  fread(wTmp, (256+1) * sizeof(WCHAR), 1, file);
-  fread(&l_h, sizeof(LEVEL_HEADER), 1, file);
+  if (fread(&pPlayer, sizeof(PLAYER_PROFILE), 1, file) != 1 ||
+      fread(wTmp, 32 * sizeof(WCHAR), 1, file) != 1 ||
+      fread(&ver, sizeof(int), 1, file) != 1 ||
+      fread(cTmp, (256+1) * sizeof(char), 1, file) != 1 ||
+      fread(&l_h, sizeof(LEVEL_HEADER), 1, file) != 1) {
+    fclose(file);
+    return 0;
+  }
 
   *pActLevel = l_h.rezerved[0];
   *pActScene = l_h.rezerved[1];
@@ -1484,7 +1530,8 @@ int lsi_Save_Exist(WCHAR * wName, char *cFile)
     return 0;
   }
 
-  getcwd(cwd,MAX_FILENAME);
+  if (getcwd(cwd,MAX_FILENAME) == NULL)
+    return 0;
 
   int ret = false;
   for(i = 0; i < c; i++) {
@@ -1503,8 +1550,11 @@ int lsi_Save_Exist(WCHAR * wName, char *cFile)
       PLAYER_PROFILE	pPlayer;
       WCHAR	wTmp[32];
     
-      fread(&pPlayer, sizeof(PLAYER_PROFILE), 1, file);
-      fread(wTmp, 32 * sizeof(WCHAR), 1, file);
+      if (fread(&pPlayer, sizeof(PLAYER_PROFILE), 1, file) != 1 ||
+	  fread(wTmp, 32 * sizeof(WCHAR), 1, file) != 1) {
+	fclose(file);
+	return 0;
+      }
       fclose(file);
 
       if(!wcscmp(pPlayer.cName, pPlayerProfile.cName) && !wcscmp(wTmp, wName))
@@ -1515,10 +1565,16 @@ int lsi_Save_Exist(WCHAR * wName, char *cFile)
       }
     }
 
-    chdir(cwd);
+    if(chdir(cwd)) {
+      kwarning(1, "Unable to chdir(%s)", cwd);
+      break;
+    }
   }
 
-  chdir(cwd);
+  if(chdir(cwd)) {
+    kwarning(1, "Unable to chdir(%s)", cwd);
+    ret = false;
+  }
 
   for(i = 0; i < c; i++)
     free(namelist[i]);
@@ -1547,7 +1603,8 @@ void delete_dir(char *p_Level_Name)
   }
   free(namelist);
 
-	chdir("..");
+	if (chdir(".."))
+		return;
 	rmdir(p_Level_Name);
 }
 #endif
@@ -1566,7 +1623,8 @@ void lsi_Save_Level(WCHAR * pwc_Level_Name, LEVELINFO * p_Level)
 
 	ZeroMemory(p_Level_Name, MAX_FILENAME);
 
-	chdir(SAVE_DIR);
+	if (chdir(SAVE_DIR))
+		return;
 
 	_strdate(pom);
 
@@ -1592,11 +1650,14 @@ void lsi_Save_Level(WCHAR * pwc_Level_Name, LEVELINFO * p_Level)
 	mkdir(p_Level_Name, DEFAULT_DIR_MASK);
 #endif
   
-	chdir(p_Level_Name);
+	if (chdir(p_Level_Name))
+		return;
 
 	lsi_copy_save(p_Level_Name, p_Level);
 
-	sprintf(buffer, "%s.lvc", p_Level_Name);
+	if (snprintf(buffer, sizeof(buffer), "%s.lvc", p_Level_Name) >=
+	    (int) sizeof(buffer))
+		return;
 
 	file = fopen(buffer, "wb");
 	if(!file)
@@ -1712,7 +1773,6 @@ void lsi_Destroy_Beetle(LEVELINFO * p_Level, int GUID, int mesh)
 int lsi_Load_Saved_Level(char *p_Level_Name, LEVELINFO * p_Level)
 {
   int real;
-  char text[MAX_FILENAME];
   char ctext[MAX_FILENAME];
   FILE *file;
   int i, j;
@@ -1730,40 +1790,46 @@ int lsi_Load_Saved_Level(char *p_Level_Name, LEVELINFO * p_Level)
 
   file = fopen(ctext, "rb");
   if (!file) {
-    sprintf(text, "Level change file %s not present", ctext);
-    kprintf(1, text);
+    kprintf(1, "Level change file %s not present", ctext);
     return -1;
   }
 
-  fread(&pProfile, sizeof(PLAYER_PROFILE), 1, file);
+  if (fread(&pProfile, sizeof(PLAYER_PROFILE), 1, file) != 1 ||
+      fread(pwc_Level_Name, 32 * sizeof(WCHAR), 1, file) != 1 ||
+      fread(&ver, sizeof(int), 1, file) != 1) {
+    kprintf(1, "Cannot read level change file %s", ctext);
+    fclose(file);
+    return -2;
+  }
+
   lsi_Add_Prifile(&pProfile);
 
-  fread(pwc_Level_Name, 32 * sizeof(WCHAR), 1, file);
-
-  fread(&ver, sizeof(int), 1, file);
   if (ver != SAVE_VER) {
     kprintf(1, "Save version mismatch! Level %s", p_Level_Name);
     fclose(file);
     return -2;
   }
 
-  fread(p_Level->cLoadedFrom, (256+1)*sizeof(char), 1, file);
-  fread(&l_h, sizeof(LEVEL_HEADER), 1, file);
+  if (fread(p_Level->cLoadedFrom, (256+1)*sizeof(char), 1, file) != 1 ||
+      fread(&l_h, sizeof(LEVEL_HEADER), 1, file) != 1 ||
+      fread(&p_Level->dwPlayTime, sizeof(DWORD), 1, file) != 1 ||
+      fread(&p_Level->iNumOfSteps, sizeof(int), 1, file) != 1) {
+    kprintf(1, "Cannot read level change file %s", ctext);
+    fclose(file);
+    return -2;
+  }
 
   memset(p_Level->Level, 0, sizeof(p_Level->Level[0])*p_Level->Size_of_Level);
 
   iActualLevel = l_h.rezerved[0];
   iActualScene = l_h.rezerved[1];
 
-  fread(&p_Level->dwPlayTime, sizeof(DWORD), 1, file);
-  fread(&p_Level->iNumOfSteps, sizeof(int), 1, file);
-
   for (i = 0; i < p_Level->Count_Of_Items; i++) {
-#ifdef _DEBUG
-    assert(fread(&b_l_d, sizeof(b_l_d), 1, file) == 1);
-#else
-    fread(&b_l_d, sizeof(b_l_d), 1, file);
-#endif
+    if (fread(&b_l_d, sizeof(b_l_d), 1, file) != 1) {
+      kprintf(1, "Cannot read level change file %s", ctext);
+      fclose(file);
+      return -2;
+    }
 
     /* p_Level->Size[0] = x
        p_Level->Size[1] = y
@@ -1812,13 +1878,24 @@ int lsi_Load_Saved_Level(char *p_Level_Name, LEVELINFO * p_Level)
     }
   }
 
-  for (i = 0; i < p_Level->Count_Of_Items; i++)
-    if (p_Level->Item[i].p_Back_Pack)
-      fread(p_Level->Item[i].p_Back_Pack, sizeof(BACK_PACK), 1, file);
+  for (i = 0; i < p_Level->Count_Of_Items; i++) {
+    if (p_Level->Item[i].p_Back_Pack) {
+      if (fread(p_Level->Item[i].p_Back_Pack, sizeof(BACK_PACK), 1, file) != 1) {
+	kprintf(1, "Cannot read level change file %s", ctext);
+	fclose(file);
+	return -2;
+      }
+    }
+  }
 
   //ulozi spodni patro fleku
-  fread(p_Level->pSquare, sizeof(SQUAREDESC),
-    p_Level->Size[0] * p_Level->Size[1], file);
+  if (fread(p_Level->pSquare, sizeof(SQUAREDESC),
+	    p_Level->Size[0] * p_Level->Size[1], file) !=
+      (size_t) (p_Level->Size[0] * p_Level->Size[1])) {
+    kprintf(1, "Cannot read level change file %s", ctext);
+    fclose(file);
+    return -2;
+  }
 
   for (i = 0; i < p_Level->Size[0] * p_Level->Size[1]; i++) {
     p_Level->pSquare[i].pExtFlek = NULL;
@@ -1835,7 +1912,11 @@ int lsi_Load_Saved_Level(char *p_Level_Name, LEVELINFO * p_Level)
 
   //ulozi flaky privazane k prvkum
   for (i = 0; i < p_Level->Count_Of_Items; i++) {
-    fread(&p_Level->Item[i].Square, sizeof(SQUAREDESC), 1, file);
+    if (fread(&p_Level->Item[i].Square, sizeof(SQUAREDESC), 1, file) != 1) {
+      kprintf(1, "Cannot read level change file %s", ctext);
+      fclose(file);
+      return -2;
+    }
     p_Level->Item[i].Square.pExtFlek = NULL;
   }
 
@@ -1906,74 +1987,57 @@ void lsi_Bind_Materials2Items(LEVELINFO * p_Level)
   }
 }
 
-void lsi_Make_Screenshot(HDC hdc, char *pFile)
+void lsi_Make_Screenshot(char *pFile)
 {
-/*
-	HDC					hdcMem;
-	BITMAPINFO			*pbmiRGB = (BITMAPINFO *) NULL; 
-	HBITMAP				hbmRGB  = (HBITMAP) NULL;
-	PBYTE				pjBitsRGB;
-	BITMAPFILEHEADER	bmpHeader;
-	FILE				*bmpFile = NULL;
+  int width, height, depth, pitch;
+  size_t size;
+  GLenum format;
+  unsigned char *pixels, *flipped;
+  SDL_Surface *p_surf;
 
-	hdcMem  = CreateCompatibleDC(hdc);
+  p_ber->p_age->graph_get()->get(&width, &height, &depth);
+  if (depth == 32)
+    format = GL_RGBA;
+  else if (depth == 24)
+    format = GL_RGB;
+  else
+    assert(0);
+  pitch = width * (depth/8);
+  size = pitch * height;
 
-	pbmiRGB = (BITMAPINFO *)LocalAlloc(LMEM_FIXED|LMEM_ZEROINIT, sizeof(BITMAPINFO) );
-	
-	if (!pbmiRGB)
-		MessageBox(NULL,"!pbmiRGB","!!!", MB_OK);
-	
-	pbmiRGB->bmiHeader.biSize	       = sizeof(BITMAPINFOHEADER);     
-	pbmiRGB->bmiHeader.biWidth		   = Xresolution;     
-	pbmiRGB->bmiHeader.biHeight        = Yresolution;     
-	pbmiRGB->bmiHeader.biPlanes        = 1;     
-	pbmiRGB->bmiHeader.biBitCount      = 24;     
-	pbmiRGB->bmiHeader.biCompression   = BI_RGB;     
-	pbmiRGB->bmiHeader.biSizeImage     = pbmiRGB->bmiHeader.biWidth
-										 * abs(pbmiRGB->bmiHeader.biHeight) * 3; 
+  // Read the pixels
+  pixels = (unsigned char *) mmalloc(size);
+  glReadPixels(0, 0, width, height, format, GL_UNSIGNED_BYTE, pixels);
 
-	hbmRGB = CreateDIBSection(hdcMem, pbmiRGB, DIB_RGB_COLORS,
-							  (PVOID *) &pjBitsRGB, NULL, 0);
-	
-	if (!hbmRGB) 
-		MessageBox(NULL,"!hbmRGB","!!!", MB_OK);
+  // We need to invert the pixels vertically because that is how BMP
+  // files are stored.
+  flipped = (unsigned char *) mmalloc(size);
+  for (int i = 0; i < height; i++) {
+    memcpy(flipped + size - ((i + 1) * pitch),
+           pixels + (i * pitch), pitch);
+  }
+  free(pixels);
 
-	if (!SelectObject(hdcMem, hbmRGB))
-		MessageBox(NULL,"!SelectObject(hdcMem, hbmRGB)","!!!", MB_OK);
+  // Create an SDL surface from the pixels
+  p_surf =
+    SDL_CreateRGBSurfaceFrom(flipped, width, height, depth, pitch,
+#if SDL_BYTEORDER == SDL_BIG_ENDIAN
+                             0xff000000,
+                             0x00ff0000,
+                             0x0000ff00,
+                             0x000000ff
+#else
+                             0x000000ff,
+                             0x0000ff00,
+                             0x00ff0000,
+                             0xff000000
+#endif
+                             );
 
-  
-	if (!BitBlt(hdcMem, 0,0, Xresolution, Yresolution, hdc, 0,0,SRCCOPY)) 
-		MessageBox(NULL,"!BitBlt","!!!", MB_OK);
+  // Save the BMP file
+  if (SDL_SaveBMP(p_surf, pFile))
+    kprintf(1, "Cannot save screenshot to %s", pFile);
 
-	pjBitsRGB = (LPBYTE) GlobalAlloc(GMEM_FIXED, pbmiRGB->bmiHeader.biSizeImage);
-
-	if (!pjBitsRGB) 
-		MessageBox(NULL,"!pjBitsRGB","!!!", MB_OK);
-
-	if (!GetDIBits(hdcMem, hbmRGB, 0, (UINT)pbmiRGB->bmiHeader.biHeight, pjBitsRGB, 
-				   (LPBITMAPINFO)pbmiRGB, DIB_RGB_COLORS))
-		MessageBox(NULL,"!GetDIBits","!!!", MB_OK);
-		
-	bmpHeader.bfType = 'MB';
-	bmpHeader.bfSize = sizeof(BITMAPINFOHEADER) + sizeof(BITMAPFILEHEADER) + pbmiRGB->bmiHeader.biSizeImage;
-	bmpHeader.bfOffBits = sizeof(BITMAPINFOHEADER) + sizeof(BITMAPFILEHEADER);
-	bmpHeader.bfReserved1 = 0;
-	bmpHeader.bfReserved2 = 0;
-
-	bmpFile = fopen(pFile,"wb");
-
-	if (!bmpFile)
-		MessageBox(NULL,"!bmpFile","!!!", MB_OK);
-
-	fwrite(&bmpHeader,sizeof(BITMAPFILEHEADER),1,bmpFile);
-	fwrite(&pbmiRGB->bmiHeader,sizeof(BITMAPINFOHEADER),1,bmpFile);
-	fwrite(pjBitsRGB,sizeof(BYTE),pbmiRGB->bmiHeader.biSizeImage,bmpFile);
-
-	fclose(bmpFile);
-
-	DeleteDC(hdcMem);          
-	DeleteObject(hbmRGB);
-	LocalFree(pbmiRGB);
-	GlobalFree((HGLOBAL)pjBitsRGB);
-*/
+  SDL_FreeSurface(p_surf);
+  free(flipped);
 }
