@@ -12,6 +12,7 @@
 #include "3d_all.h"
 
 #include "Object.h"
+#include "Setup.h"
 
 #include "Berusky_universal.h"
 #include "Berusky3d_castice.h"
@@ -108,28 +109,24 @@ void ber_uloz_lookup(G_KONFIG * p_ber)
       bmp_putpixel(p_bmp2, x, y, barva);
     }
   }
-  bmp_uloz("e:\\linear.bmp", p_bmp1);
-  bmp_uloz("e:\\kvad.bmp", p_bmp2);
+  bmp_uloz("e:\\linear.png", p_bmp1);
+  bmp_uloz("e:\\kvad.png", p_bmp2);
 }
 
 void ber_rekonfiguruj_hra(G_KONFIG * p_ber)
 {
   char pom[200];
 
-  p_ber->conf_start_zpruhlednovac =
-    GetPrivateProfileInt("hra", "start_zpruhlednovani", 0, ini_file);
-  p_ber->conf_start_zvyraznovac =
-    GetPrivateProfileInt("hra", "start_zvyraznovani", 0, ini_file);
   p_ber->conf_ovladani_rohy =
-    GetPrivateProfileInt("hra", "ovladani_rohy", 1, ini_file);
+    GetPrivateProfileInt("game", "ovladani_rohy", 1, ini_file);
   p_ber->conf_ovladani_rohy_default =
-    GetPrivateProfileInt("hra", "ovladani_rohy_default", 0, ini_file);
+    GetPrivateProfileInt("game", "ovladani_rohy_default", 0, ini_file);
   p_ber->conf_ovladani_rohy_smer =
-    GetPrivateProfileInt("hra", "ovladani_rohy_smer", 0, ini_file);
-  GetPrivateProfileString("hra", "ovladani_rohy_rychlost", "0.5", pom, 200,
+    GetPrivateProfileInt("game", "ovladani_rohy_smer", 0, ini_file);
+  GetPrivateProfileString("game", "ovladani_rohy_rychlost", "0.5", pom, 200,
     ini_file);
   p_ber->conf_ovladani_rohy_rychlost = (float) atof(pom);
-  GetPrivateProfileString("hra", "p_kamera_radius", "4.0", pom, 200,
+  GetPrivateProfileString("game", "p_kamera_radius", "4.0", pom, 200,
     ini_file);
   p_ber->conf_pruhledna_kamera_radius = (float) atof(pom);
 }
@@ -138,20 +135,19 @@ void ber_rekonfiguruj_3D(G_KONFIG * p_ber)
 {
   char pom[200];
 
-  hwconf.ditering = GetPrivateProfileInt("hra", "ditering", 0, ini_file);
+  hwconf.ditering = GetPrivateProfileInt("game", "ditering", 0, ini_file);
+  
+  p_ber->conf_animace_okoli = GetPrivateProfileInt("game", "animace_okoli", 1, ini_file);
+  p_ber->conf_caustic = GetPrivateProfileInt("game", "scene_materialy", 1, ini_file);
 
-  p_ber->conf_zrcadlo = GetPrivateProfileInt("hra", "zrcado_aktivni", 1, ini_file);
-  p_ber->conf_animace_okoli = GetPrivateProfileInt("hra", "animace_okoli", 1, ini_file);
-  p_ber->conf_caustic = GetPrivateProfileInt("hra", "scene_materialy", 1, ini_file);
-
-  GetPrivateProfileString("textury", "text_ostrost", "0.0", pom, 200, ini_file);
+  GetPrivateProfileString("textures", "text_ostrost", "0.0", pom, 200, ini_file);
   txconf.text_ostrost = (float) atof(pom);
 
   // 0/1 - on/off
   // 1,2,3 - hi/mid/low
-  p_ber->conf_dyn_light = GetPrivateProfileInt("hra", "light_dyn", 1, ini_file);
+  p_ber->conf_dyn_light = GetPrivateProfileInt("game", "light_dyn", 1, ini_file);
   if (p_ber->conf_dyn_light)
-    p_ber->conf_dyn_rychlost = GetPrivateProfileInt("hra", "light_dyn_speed", 1, ini_file);
+    p_ber->conf_dyn_rychlost = GetPrivateProfileInt("game", "light_dyn_speed", 1, ini_file);
   else
     p_ber->conf_dyn_rychlost = LIGHT_NIC;
 }
@@ -173,7 +169,7 @@ void ber_rekonfiguruj_3D_aplikuj(G_KONFIG * p_ber)
 // taky float lok-up tabulka !!!
 void ber_konfiguruj_berusky(G_KONFIG * p_ber)
 {
-  char pom[200];
+  char pom[MAX_FILENAME];
   int i;
 
   p_ber->conf_barva_pozadi = 0xff000000;
@@ -198,69 +194,130 @@ void ber_konfiguruj_berusky(G_KONFIG * p_ber)
   for (i = 0; i < SIN_TABLE_SIZE; i++)
     p_ber->sinus_table[i] = sinf(DEG2RAD(i));
 
-  GetPrivateProfileString("game", "prvky_dir", ".", p_ber->dir.prvky_dir, MAX_FILENAME, ini_file);
+	GetPrivateProfileString("install","save_dir","/", p_ber->dir.save_dir, MAX_FILENAME, ini_file);
+  working_file_translate(p_ber->dir.save_dir, MAX_FILENAME);
+  
+	GetPrivateProfileString("install","profile_dir","/", p_ber->dir.profile_dir, MAX_FILENAME, ini_file);
+  working_file_translate(p_ber->dir.profile_dir, MAX_FILENAME);
+
+  // Load root of the data files
+	GetPrivateProfileString("install","game_root","/", p_ber->dir.game_root_dir, MAX_FILENAME, ini_file);
+  working_file_translate(p_ber->dir.game_root_dir, MAX_FILENAME);
+
+	GetPrivateProfileString("files","bitmap_dir","/", p_ber->dir.bitmap_dir, MAX_FILENAME, ini_file);
+  working_file_translate(p_ber->dir.bitmap_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.bitmap_dir, p_ber->dir.game_root_dir);
+
+  GetPrivateProfileString("files", "prvky_dir", ".", p_ber->dir.prvky_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.prvky_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.prvky_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "level_dir", ".", p_ber->dir.level_dir, MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "level_dir", ".", p_ber->dir.level_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.level_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.level_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "out_dir", ".", p_ber->dir.out_dir, MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "out_dir", ".", p_ber->dir.out_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.out_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.out_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "material_dir", ".", p_ber->dir.material_dir, MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "material_dir", ".", p_ber->dir.material_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.material_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.material_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "data_dir", ".", p_ber->dir.data_dir, MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "data_dir", ".", p_ber->dir.data_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.data_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.data_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "game_level_dir", ".", p_ber->dir.game_level_dir, MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "game_level_dir", ".", p_ber->dir.game_level_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.game_level_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.game_level_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "game_data_dir", ".", p_ber->dir.game_data_dir, MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "game_data_dir", ".", p_ber->dir.game_data_dir, MAX_FILENAME, ini_file);
   working_file_translate(p_ber->dir.game_data_dir, MAX_FILENAME);
+  root_dir_attach(p_ber->dir.game_data_dir, p_ber->dir.game_root_dir);
   
-  GetPrivateProfileString("game", "texture_dir", ".",  p_ber->tdir.texture_dir[0], MAX_FILENAME, ini_file);
+  GetPrivateProfileString("files", "texture_dir", ".",  p_ber->tdir.texture_dir[0], MAX_FILENAME, ini_file);
   working_file_translate(p_ber->tdir.texture_dir[0], MAX_FILENAME);
+  root_dir_attach(p_ber->tdir.texture_dir[0], p_ber->dir.game_root_dir);
 
   for (i = 0; i < TEXT_DIRS; i++) {
-    sprintf(pom, "texture_dir%d", i);
-    GetPrivateProfileString("game", pom, "", p_ber->tdir.texture_dir[i],  MAX_FILENAME, ini_file);
+    char old[200];
+    int old_class;
+    const struct {
+      const char *const dir;
+      const int dir_class;
+    } defaults[TEXT_DIRS] = {
+        { "textures/general",  1 },
+        { "textures/stul",     3 },
+        { "textures/balkon",   3 },
+        { "textures/kanal",    3 },
+        { "textures/voda",     3 },
+        { "textures/silnice",  3 },
+        { "textures/zahrada",  3 },
+        { "textures/bazina",   3 },
+        { "textures/les",      3 },
+        { "textures/tutorial", 3 },
+        { "textures/strom",    3 },
+        { "textures/detska",   3 },
+        { "textures/items",    2 }
+    };
+    const char *default_dir = defaults[i].dir;
+    const int default_class = defaults[i].dir_class;
+
+    if (!default_dir)
+      default_dir = "";
+
+    snprintf(pom, sizeof(pom), "texture_dir%d", i);
+    snprintf(old, sizeof(old), "texture_file%d", i);
+    get_dir_setting("files", pom, old, default_dir,
+                    p_ber->tdir.texture_dir[i],
+                    MAX_FILENAME, ini_file);
+
     working_file_translate(p_ber->tdir.texture_dir[i], MAX_FILENAME);
+    root_dir_attach(p_ber->tdir.texture_dir[i], p_ber->dir.game_root_dir);
 
     if (p_ber->tdir.texture_dir[i][0] == '.' && !p_ber->tdir.texture_dir[i][1])
       p_ber->tdir.texture_dir[i][0] = 0;
 
-    sprintf(pom, "texture_dir%d_class", i);
-    p_ber->tdir.texture_dir_class[i] = GetPrivateProfileInt("game", pom, 0, ini_file);
+    snprintf(pom, sizeof(pom), "texture_dir%d_class", i);
+    p_ber->tdir.texture_dir_class[i] =
+      GetPrivateProfileInt("files", pom, default_class, ini_file);
+
+    // Remove the old setting if it's the default.
+    snprintf(old, sizeof(old), "texture_file%d_class", i);
+    old_class = GetPrivateProfileInt("files", old, 0, ini_file);
+    if (old_class == default_class) {
+      kprintf(TRUE, "Removing obsolete setting with default value: %s=%d",
+              old, old_class);
+      WritePrivateProfileString("files", old, NULL, ini_file);
+    }
   }
 
-  for (i = 0; i < TEXT_DIRS; i++) {
-    sprintf(pom, "texture_file%d", i);
-    GetPrivateProfileString("game", pom, "", p_ber->tdir.texture_file[i],  MAX_FILENAME, ini_file);
-    working_file_translate(p_ber->tdir.texture_file[i], MAX_FILENAME);
-  
-    if (p_ber->tdir.texture_file[i][0] == '.' && !p_ber->tdir.texture_file[i][1])
-      p_ber->tdir.texture_file[i][0] = 0;
-    
-    sprintf(pom, "texture_file%d_class", i);
-    p_ber->tdir.texture_file_class[i] = GetPrivateProfileInt("game", pom, 0, ini_file);
-  }
+  // 'data_pak' is no longer used, and there is no corresponding
+  // 'data_dir'. (Well, there is a 'data_dir', but that's for
+  // something else). So just remove 'data_pak' from the INI file if
+  // it exists. However, only remove it if it's equal to "data.pak",
+  // because otherwise the user has changed it and might want to
+  // remember that setting.
+  GetPrivateProfileString("files", "data_pak", "", pom, sizeof(pom), ini_file);
+  if (!strcmp(pom, "data.pak"))
+    WritePrivateProfileString("files", "data_pak", NULL, ini_file);
 
-	GetPrivateProfileString("game","save_dir","c:\\", p_ber->dir.save_dir, MAX_FILENAME, ini_file);
-  working_file_translate(p_ber->dir.save_dir, MAX_FILENAME);
+	GetPrivateProfileString("soundengine","sound_dir","/",p_ber->dir.sound_dir,MAX_FILENAME,ini_file);
+  working_file_translate(p_ber->dir.sound_dir,MAX_FILENAME);
+  root_dir_attach(p_ber->dir.sound_dir, p_ber->dir.game_root_dir);
+  adas_set_sound_dir(p_ber->dir.sound_dir);
 
-	GetPrivateProfileString("game","bitmap_dir","c:\\", p_ber->dir.bitmap_dir, MAX_FILENAME, ini_file);
-  working_file_translate(p_ber->dir.bitmap_dir, MAX_FILENAME);
-
-	GetPrivateProfileString("game","profile_dir","c:\\", p_ber->dir.profile_dir, MAX_FILENAME, ini_file);
-  working_file_translate(p_ber->dir.profile_dir, MAX_FILENAME);
+	GetPrivateProfileString("soundengine","music_dir","/",p_ber->dir.music_dir,MAX_FILENAME,ini_file);
+  working_file_translate(p_ber->dir.music_dir,MAX_FILENAME);
+  root_dir_attach(p_ber->dir.music_dir, p_ber->dir.game_root_dir);
 
   p_ber->conf_barva_pozadi_pouzit_default = 
   p_ber->conf_barva_pozadi_pouzit = 
-  GetPrivateProfileInt("hra", "barva_pozadi", 0, ini_file);
+  GetPrivateProfileInt("game", "barva_pozadi", 0, ini_file);
 
   p_ber->conf_extra_light_vertex = 
-  GetPrivateProfileInt("hra", "extra_light_vertex", 1, ini_file);
+  GetPrivateProfileInt("game", "extra_light_vertex", 1, ini_file);
 
   ber_rekonfiguruj_hra(p_ber);
   ber_rekonfiguruj_3D(p_ber);
@@ -445,12 +502,12 @@ void ber_init_level(G_KONFIG * p_ber)
 
   /* Zpruhlednovani prvku pred kamerou
    */
-  if (p_ber->conf_start_zpruhlednovac)
+  if (setup.bugs_highlight)
     kom_zpruhlednovat_prvky(TRUE);
 
   /* Nahodi zvyraznovani prvku
    */
-  p_ber->conf_full_light = !p_ber->conf_start_zvyraznovac;
+  p_ber->conf_full_light = !setup.items_highlight;
   ber_prikaz_zvyraznovac(p_ber);
 
   /* Vypne kurzor
@@ -806,7 +863,6 @@ void ber_test_animaci(G_KONFIG * p_ber)
          animace)
        */
       j = 0;
-      chani_zarad(p_handle[j]);
 
       /* Zarazenou animaci spustim
        */
