@@ -9,6 +9,7 @@
 #include "3d_all.h"
 
 #include "Object.h"
+#include "Setup.h"
 
 #include "Berusky_universal.h"
 #include "Berusky3d_castice.h"
@@ -26,6 +27,15 @@ extern G_KONFIG ber, *p_ber;
 
 float kom_load_progres;         // aktualni tick
 
+// For restoring mirror effects.
+int zrc_akt;
+int *p_poly_indices;
+struct poly_save_data {
+  dword kflag, m1flag;
+  int alfa_state;
+} *p_poly_data;
+unsigned int n_poly_indices, n_poly_data;
+
 /*
   Load levelu
 */
@@ -372,6 +382,12 @@ void kom_zrus_level(int restart)
     p_ber->p_poly = NULL;
   }
 
+  if (zrc_akt) {
+    zrc_akt = FALSE;
+    free(p_poly_indices);
+    free(p_poly_data);
+  }
+
   kprintf(TRUE, "Kom_zrus_level - mesh...");
 
   /* Zrus meshe
@@ -392,18 +408,18 @@ void kom_zrus_level(int restart)
     }
   }
 
+  kprintf(TRUE, "Kom_zrus_level - materialy...");
+
+  /* Zrus materiay + smaterial
+   */
+  for (i = 0; i < MAX_CELKEM_MATERIALU; i++) {
+    if (p_ber->p_mat[i])
+      zrus_material(p_ber->p_mat + i);
+  }
+  p_ber->p_smat = NULL;
+
   if (!restart) {
 
-    kprintf(TRUE, "Kom_zrus_level - materialy...");
-
-    /* Zrus materiay + smaterial
-     */
-    for (i = 0; i < MAX_CELKEM_MATERIALU; i++) {
-      if (p_ber->p_mat[i])
-        zrus_material(p_ber->p_mat + i);
-    }
-    p_ber->p_smat = NULL;
-
     kprintf(TRUE, "Kom_zrus_level - textury...");
 
     /* Zrus textury
@@ -473,7 +489,8 @@ void kom_zrus_level(int restart)
      STATIC_LIGHT         p_slight[MAX_FLARE_SVETEL];
    */
   p_ber->slightnum = 0;
-  memset(p_ber->p_slight, 0, sizeof(p_ber->p_slight[0]) * MAX_FLARE_SVETEL);
+  memset((void *) p_ber->p_slight, 0,
+	 sizeof(p_ber->p_slight[0]) * MAX_FLARE_SVETEL);
 
   kprintf(TRUE, "Kom_zrus_level - dyn svetla...");
 
@@ -484,7 +501,8 @@ void kom_zrus_level(int restart)
       sdla_animaci_zrus_online(&p_ber->p_dlight[i].an);
     }
   }
-  memset(p_ber->p_dlight, 0, sizeof(p_ber->p_dlight[0]) * MAX_FLARE_SVETEL);
+  memset((void *) p_ber->p_dlight, 0,
+	 sizeof(p_ber->p_dlight[0]) * MAX_FLARE_SVETEL);
 
   kprintf(TRUE, "Kom_zrus_level - extra-dyn svetla...");
 
@@ -495,7 +513,8 @@ void kom_zrus_level(int restart)
       sdla_animaci_zrus_online(&p_ber->p_edlight[i].an);
     }
   }
-  memset(p_ber->p_edlight, 0, sizeof(p_ber->p_edlight[0]) * MAX_FLARE_SVETEL);
+  memset((void *) p_ber->p_edlight, 0,
+	 sizeof(p_ber->p_edlight[0]) * MAX_FLARE_SVETEL);
 
   kprintf(TRUE, "Kom_zrus_level - sim-animaci...");
 
@@ -509,8 +528,10 @@ void kom_zrus_level(int restart)
       zrus_tri_vnitrek(p_ber->p_track + i);
     }
   }
-  memset(p_ber->p_sim, 0, sizeof(p_ber->p_sim[0]) * MAX_SIM_ANIMACI);
-  memset(p_ber->p_track, 0, sizeof(p_ber->p_track[0]) * MAX_SIM_ANIMACI);
+  memset((void *) p_ber->p_sim, 0,
+	 sizeof(p_ber->p_sim[0]) * MAX_SIM_ANIMACI);
+  memset((void *) p_ber->p_track, 0,
+	 sizeof(p_ber->p_track[0]) * MAX_SIM_ANIMACI);
 
   kprintf(TRUE, "Kom_zrus_level - hit-list...");
 
@@ -534,7 +555,7 @@ void kom_zrus_level(int restart)
       p_poly = p_tmp;
     }
     p_ber->zrc_akt = FALSE;
-    memset(&p_ber->zrc, 0, sizeof(p_ber->zrc));
+    memset((void *) &p_ber->zrc, 0, sizeof(p_ber->zrc));
   }
 
   kprintf(TRUE, "Kom_zrus_level - obb a renderlist...");
@@ -542,7 +563,7 @@ void kom_zrus_level(int restart)
   /* Smaze obalky + poly render list
    */
   obbtree_zrus(&p_ber->obbtree);
-  memset(&p_ber->obbtree, 0, sizeof(p_ber->obbtree));
+  memset((void *) &p_ber->obbtree, 0, sizeof(p_ber->obbtree));
   ber_poly_render_list_zrus(p_ber);
 
   memset(p_ber->p_dyn_meshlist, 0,
@@ -570,7 +591,8 @@ void kom_zrus_level(int restart)
    */
   p_ber->p_gkanim = NULL;
   p_ber->gknum = 0;
-  memset(p_ber->p_gkzas, 0, sizeof(p_ber->p_gkzas[0]) * MAX_RUN_ANIMACI);
+  memset((void *) p_ber->p_gkzas, 0,
+	 sizeof(p_ber->p_gkzas[0]) * MAX_RUN_ANIMACI);
 
   kprintf(TRUE, "Kom_zrus_level - animace chapadel...");
 
@@ -607,29 +629,102 @@ int ber_test_zrcadla(void)
 {
   ZDRCADLO_DESC_POLY *p_poly = p_ber->zrc.p_poly;
   int mat;
+  unsigned int i, j;
+
+  // If `zrc_akt' is TRUE, we've already disabled mirror effects.
+  if (zrc_akt && !setup.mirror_effects)
+    return (TRUE);
 
   // Ma se vypnout zrcadlo
-  if (p_ber->zrc_akt && !p_ber->conf_zrcadlo) {
+  if (p_ber->zrc_akt && !setup.mirror_effects) {
+    zrc_akt = TRUE;
     p_ber->zrc_akt = FALSE;
+
+    // Prepare for saving the old data to restore mirror effects
+    // later.
+    n_poly_indices = n_poly_data = 0;
     while (p_poly) {
+      n_poly_indices++;
+      if (p_poly->poly != K_CHYBA && p_ber->polynum > p_poly->poly)
+        n_poly_data++;
+      p_poly = p_poly->p_next;
+    }
+    p_poly = p_ber->zrc.p_poly;
+
+    p_poly_indices = (int *)
+      mmalloc(sizeof(*p_poly_indices) * n_poly_indices);
+    p_poly_data = (struct poly_save_data *)
+      mmalloc(sizeof(*p_poly_data) * n_poly_data);
+
+    for (i = j = 0; p_poly; i++) {
+      // Save the index for later restoring of mirror effects.
+      assert(i < n_poly_indices);
+      p_poly_indices[i] = p_poly->poly;
+
       if (p_poly->poly != K_CHYBA && p_ber->polynum > p_poly->poly) {
+        // Save the flags.
+        assert(j < n_poly_data);
+        p_poly_data[j].kflag =
+          p_ber->p_poly[p_poly->poly].kflag & KONT_ZRCADLO;
+        p_poly_data[j].m1flag =
+          p_ber->p_poly[p_poly->poly].m1flag & MAT_PRUHLEDNY;
+
         p_ber->p_poly[p_poly->poly].kflag &= ~KONT_ZRCADLO;
         p_ber->p_poly[p_poly->poly].m1flag &= ~MAT_PRUHLEDNY;
 
         mat = p_ber->p_poly[p_poly->poly].material;
+
+        // Save `alfa_state'.
+        p_poly_data[j].alfa_state = p_ber->p_mat[mat]->alfa_state;
+
         p_ber->p_mat[mat]->alfa_state = FALSE;
 
-        p_poly->poly = K_CHYBA;
-      }
-      else if (p_ber->polynum <= p_poly->poly) {
-        p_poly->poly = K_CHYBA;
+        j++;
       }
+
+      p_poly->poly = K_CHYBA;
       p_poly = p_poly->p_next;
     }
+
+    assert(i == n_poly_indices);
+    assert(j == n_poly_data);
+
     return (TRUE);
   }
-  else
-    return (FALSE);
+  else if (zrc_akt) {
+    // Restore mirror effects.
+    p_ber->zrc_akt = TRUE;
+    for (i = j = 0; i < n_poly_indices; i++) {
+      assert(p_poly);
+      p_poly->poly = p_poly_indices[i];
+
+      if (p_poly->poly != K_CHYBA && p_ber->polynum > p_poly->poly) {
+        assert(j < n_poly_data);
+        p_ber->p_poly[p_poly->poly].kflag |= p_poly_data[j].kflag;
+        p_ber->p_poly[p_poly->poly].m1flag |= p_poly_data[j].m1flag;
+
+        mat = p_ber->p_poly[p_poly->poly].material;
+        p_ber->p_mat[mat]->alfa_state = p_poly_data[j].alfa_state;
+
+        j++;
+      }
+
+      p_poly = p_poly->p_next;
+    }
+
+    assert(i == n_poly_indices);
+    assert(j == n_poly_data);
+
+    // Clean up.
+    zrc_akt = FALSE;
+    free(p_poly_indices);
+    free(p_poly_data);
+
+    // Make sure all reflections are updated.
+    p_ber->kamera.zmena = TRUE;
+  }
+
+  return (FALSE);
 }
 
 void kom_load_sys_material(int i)
@@ -638,7 +733,7 @@ void kom_load_sys_material(int i)
   char pom[200];
 
   sprintf(pom, "system_material_%d", i);
-  GetPrivateProfileString("game", pom, "", file, 200, ini_file);
+  GetPrivateProfileString("files", pom, "", file, 200, ini_file);
   if (file[0]) {
     kprintf(1, "Load system materials '%s' from %s....", pom, file);
     if (!ber_nahraj_materialy(p_ber, file, p_ber->dir.material_dir))
@@ -667,7 +762,7 @@ int ber_vyber_berusky(BUNKA_LEVELU_DISK * p_disk, int num,
   }
 
   if (berusek > 5) {
-    kprintfe(TRUE, "Mas tam moc berusek!!! %d", berusek);
+    kprintfe(TRUE, "Too many bugs in the level!!! Bugs number %d", berusek);
   }
 
   return (berusek);
@@ -694,19 +789,21 @@ void kom_load_level(char *p_file, int zmen_dir, int restart,
   kom_posun_slider();
 
   // Load systemovych materialu
-  getcwd(dir, 200);
+  if (getcwd(dir, 200) == NULL)
+    konec(TRUE);
 
   for (i = 0; i < 10; i++)
     kom_load_sys_material(i);
 
   if (zmen_dir) {
-    chdir((p_ber->dir.game_level_dir));
+    if (chdir(p_ber->dir.game_level_dir))
+      konec(TRUE);
     strcpy(dir, p_file);
     zamen_koncovku(dir, "");
   }
 
-  chdir((dir));
-  getcwd(dir, 200);
+  if (chdir(dir) || getcwd(dir, 200) == NULL)
+    konec(TRUE);
   strcpy(file, p_file);
 
   kom_posun_slider();
@@ -748,6 +845,8 @@ void kom_load_level(char *p_file, int zmen_dir, int restart,
 
   kom_posun_slider();
 
+  zrc_akt = FALSE;
+
   if (ber_test_zrcadla())
     kprintf(TRUE, "Mirror off");
   else
@@ -933,6 +1032,9 @@ void kom_flek_zmen(FlekHandle fh, BOD * p_p, BOD * p_nx, BOD * p_nz, float dx,
   p_flek->material = material;
   p_flek->p = *p_p;
 
+  // Eliminate y-axis fighting
+  p_flek->p.y += 0.02;
+
   p_flek->nx.x = p_nx->x * dx;
   p_flek->nx.y = p_nx->y * dx;
   p_flek->nx.z = p_nx->z * dx;
