@@ -7,7 +7,7 @@
 // vynuluje matici
 inline GLMATRIX *zero_matrix(GLMATRIX * m)
 {
-  memset(m, 0, sizeof(*m));
+  memset((void *) m, 0, sizeof(*m));
   return (m);
 }
 
@@ -352,8 +352,8 @@ inline GLMATRIX *invert_matrix(GLMATRIX * src, GLMATRIX * inv)
   det = inv->_11 * src->_11 + inv->_12 * src->_21 + inv->_13 * src->_31;
   if (det == 0.0f) {
     // we're not very happy with the source matrix ;-)
-    memset(inv, 0, sizeof(*inv));
-    return (NULL);
+    init_matrix(inv);
+    return inv;
   }
 
   inv->_21 = -src->_21 * src->_33 + src->_23 * src->_31;
@@ -378,7 +378,7 @@ inline GLMATRIX *invert_matrix(GLMATRIX * src, GLMATRIX * inv)
   inv->_24 = 0;
   inv->_34 = 0;
   inv->_44 = 1;
-  
+
   return (inv);
 }
 
@@ -400,9 +400,10 @@ inline GLMATRIX *invert_matrix_copy(GLMATRIX * src, GLMATRIX * inv)
   m._12 = -src->_12 * src->_33 + src->_13 * src->_32;
   m._13 = src->_12 * src->_23 - src->_13 * src->_22;
   det = m._11 * src->_11 + m._12 * src->_21 + m._13 * src->_31;
-  if (det == 0) {
-    memset(inv, 0, sizeof(*inv));
-    return (NULL);
+  if (det == 0.0f) {
+    // we're not very happy with the source matrix ;-)
+    init_matrix(inv);
+    return inv;
   }
   m._21 = -src->_21 * src->_33 + src->_23 * src->_31;
   m._22 = src->_11 * src->_33 - src->_13 * src->_31;
