@@ -27,6 +27,7 @@
  */
 
 #include <sys/time.h>
+#include <errno.h>
 
 #include "3d_all.h"
 #include "ini.h"
@@ -78,7 +79,7 @@ unsigned int timeGetTime(void)
 //-----------------------------------------------------------------------------
 // ddx2 interface
 //-----------------------------------------------------------------------------
-extern APAK_HANDLE *pBmpArchive;
+extern char pBmpDir[MAX_FILENAME];
 DeviceHandle ddxDevice;
 DeviceHandle ddxCursorDevice;
 int i_CursorDDX = 0;
@@ -216,7 +217,7 @@ int ddxLoadList(char *pFileName, int bProgress)
   }
 
   kprintf(1, "Kofola: - Load bitmap pro herni menu");
-  ddx2LoadList(pFileName, pBmpArchive, BITMAP_DIR);
+  ddx2LoadList(pFileName, pBmpDir, BITMAP_DIR);
 
   if (bProgress) {
     ddxSetCursor(1);
@@ -226,9 +227,9 @@ int ddxLoadList(char *pFileName, int bProgress)
   return (1);
 }
 
-int ddxLoadBitmap(char *pFileName, APAK_HANDLE * pHandle)
+int ddxLoadBitmap(char *pFileName, char* pDirName)
 {
-  return (ddx2LoadBitmap(pFileName, pHandle));
+  return (ddx2LoadBitmap(pFileName, pDirName));
 }
 
 int ddxCreateSurface(int x, int y, int idx)
@@ -533,46 +534,96 @@ void dbgprintf(char *p_tmp, ...)
 {
 }
 
-int CP1250ToWideChar(char *lpMultiByteStr, size_t multiByteStrLen,
-                     char *lpWideCharStr, size_t wideCharLen)
+// Convert any encoding to another encoding, returning the number of
+// bytes in the encoded string.
+size_t ConvertEncoding(const char *fromEncoding,
+                       const char *toEncoding,
+                       char *lpFromStr, size_t cbFrom,
+                       char *lpToStr, size_t cbTo)
 {
-  iconv_t cd = iconv_open("UTF32", "WINDOWS-1250");
-  if (cd == (iconv_t)-1)
-    return(-1);
-  
-  int ret = iconv(cd, 
-                  &lpMultiByteStr, &multiByteStrLen,
-                  &lpWideCharStr, &wideCharLen);
+  iconv_t cd;
+  size_t ret;
 
-  iconv_close(cd);
-  assert(wideCharLen);
-  return(ret != -1 && wideCharLen);
+  cd = iconv_open(toEncoding, fromEncoding);
+  if (cd == (iconv_t)-1)
+    return ((size_t)-1);
+
+  if (!lpToStr) {
+    size_t count = 0;
+
+    do {
+      // Use a temporary buffer in place of lpToStr.
+      char buf[4096];           // An arbitrary size
+      char *p_buf = buf;
+      size_t buf_size = sizeof(buf);
+
+      ret = iconv(cd, &lpFromStr, &cbFrom, &p_buf, &buf_size);
+      if (ret == (size_t)-1 && errno != E2BIG)
+        break;
+
+      count += sizeof(buf) - buf_size;
+    } while (ret == (size_t)-1);
+
+    if (ret != (size_t)-1)
+      ret = count;
+  }
+  else {
+    size_t oldCbTo = cbTo;
+    ret = iconv(cd, &lpFromStr, &cbFrom, &lpToStr, &cbTo);
+    if (ret != (size_t)-1)
+      ret = oldCbTo - cbTo;
+  }
+
+  if (iconv_close(cd))
+    return ((size_t)-1);
+
+  return (ret);
 }
 
 // Converts UTF-8 to wide-char string
 int MultiByteToWideChar(int CodePage, int dwFlags, char *lpMultiByteStr,
                         int cbMultiByte, WCHAR * lpWideCharStr, int cchWideChar)
 {
-  int ret = mbstowcs(lpWideCharStr, lpMultiByteStr, cchWideChar);
-  if(ret != -1) {
-    assert(ret && ret < cchWideChar);
+  size_t ret;
+  size_t n_converted;
+
+  cbMultiByte = strnlen(lpMultiByteStr, cbMultiByte);
+  ret =
+    ConvertEncoding("UTF8", "WCHAR_T",
+                    lpMultiByteStr, cbMultiByte,
+                    (char *)lpWideCharStr, cchWideChar*sizeof(WCHAR));
+  if (ret == (size_t)-1)
+    return -1;
+
+  n_converted = ret / sizeof(WCHAR);
+  if (lpWideCharStr) {
+    assert(n_converted < (size_t)cchWideChar);
+    lpWideCharStr[n_converted] = L'\0';
   }
-  else {
-    // invalid multi-byte sequence has been detected - try to convert
-    // from Windows-1250 codepage      
-    ret = CP1250ToWideChar(lpMultiByteStr, 
-                           cbMultiByte, 
-                           (char *)lpWideCharStr,
-                           cchWideChar*sizeof(WCHAR));    
-  }
-  return (ret);
+
+  return n_converted;
 }
 
 int WideCharToMultiByte(int CodePage, int dwFlags, wchar_t * lpWideCharStr,
                         int cchWideChar, char *lpMultiByteStr, int cbMultiByte, 
                         char *lpDefaultChar, int *lpUsedDefaultChar)
 {
-  return (wcstombs(lpMultiByteStr, lpWideCharStr, cbMultiByte));
+  size_t ret;
+
+  cchWideChar = wcsnlen(lpWideCharStr, cchWideChar);
+  ret =
+    ConvertEncoding("WCHAR_T", "UTF8",
+                    (char *)lpWideCharStr, cchWideChar*sizeof(WCHAR),
+                    lpMultiByteStr, cbMultiByte);
+  if (ret == (size_t)-1)
+    return -1;
+
+  if (lpMultiByteStr) {
+    assert(ret < (size_t)cbMultiByte);
+    lpMultiByteStr[ret] = '\0';
+  }
+
+  return ret;
 }
 
 void ShowCursor(bool state)
@@ -596,8 +647,7 @@ void GetPrivateProfileString(const char *lpAppName,     // section name
   const char *lpDefault,        // default key value
   char *lpReturnedString, 
   int nSize, 
-  const char *lpFileName     // ini file name
-  )
+  const char *lpFileName)       // ini file name
 { 
   ini_read_string_section(lpFileName, lpAppName, lpKeyName, 
                           lpReturnedString, nSize, lpDefault);
@@ -676,7 +726,8 @@ char current_working_dir_file[MAX_FILENAME];
 
 void working_dir_init(void)
 {
-  getcwd(current_working_dir, MAX_FILENAME);
+  char *ret = getcwd(current_working_dir, MAX_FILENAME);
+  assert(ret);
 }
 
 char *working_file_get(const char *p_file)
@@ -697,6 +748,15 @@ void working_file_translate(char *p_file, int size)
   strcpy(p_file, current_working_dir_file);
 }
 
+void root_dir_attach(char *p_dir, char *p_root_dir)
+{
+  char tmp[MAX_FILENAME];
+  strcpy(tmp, p_root_dir);
+  strcat(tmp, "/");
+  strcat(tmp, p_dir);
+  strcpy(p_dir, tmp);
+}
+
 void GetFileSize(FILE *f, dword *size)
 {
   *size = file_size_get(f);
